import hashlib

from sdfs.utils import keyify

class MetadataMixin:
    
    """
    A mixing for plugins that enables them to easily set new metadata for an
    item correctly.
    """
    
    def set_metadata(self, item_type, virtual_path, key, value):
        """
        Sets new metadata for a given item.
        
        Parameters:
        
        - `item_type`: Type of item.
        - `virtual_path`: Virtual path to the item.
        - `key`: Key to set value for.
        - `value`: Value to set.
        """
        db_key = keyify(item_type, virtual_path)
        val = self.db[db_key]
        key = 'plugin:%s:%s' % (self.name, key)
        val[key] = value
        self.db[db_key] = val