import uuid
import os
import urllib
import types
import hashlib
import base64
from datetime import datetime

from zope.interface import implements
from twisted.plugin import IPlugin
from twisted.internet import reactor
from twisted.web import static, server, resource, http
from twisted.python import log, randbytes

from sdfs.iurihandler import IURIHandler
from sdfs.exceptions import FileOrFolderNotFoundException


class FilelikeObjectResource(static.File):
    isLeaf = True
    contentType = None
    fileObject = None
    encoding = 'bytes'
    
    def __init__(self, fileObject, size, contentType='bytes'):
        self.contentType = contentType
        self.fileObject = fileObject
        self.fileSize = size
        resource.Resource.__init__(self)
    
    def getFileSize(self):
        return self.fileSize
    
    def render_GET(self, request):
        """
        Begin sending the contents of this L{File} (or a subset of the
        contents, based on the 'range' header) to the given request.
        """
        request.setHeader('accept-ranges', 'bytes')

        producer = self.makeProducer(request, self.fileObject)

        if request.method == 'HEAD':
            return ''
        
        producer.start()
        # and make sure the connection doesn't get closed
        return server.NOT_DONE_YET
    render_HEAD = render_GET


class HTTPServer(resource.Resource):
    filelist = {}
    
    def getChild(self, path, request):
        if path not in self.filelist or self.filelist[path][2] and self.filelist[path][2] != request.getClientIP():
            return resource.NoResource()
        
        if isinstance(self.filelist[path][0], types.StringTypes):
            f = static.File(self.filelist[path][0], defaultType='application/octet-stream')
            f.isLeaf = True
        else:
            obj, size = self.filelist[path][0].get_object()
            f = FilelikeObjectResource(obj, size)
        return f


class HTTPURIHandler(object):
    implements(IPlugin, IURIHandler)
    
    scheme = 'http'
    ip = None
    port = None

    def prepare_serve(self, f, ip=None):
        id = base64.urlsafe_b64encode(randbytes.RandomFactory().secureRandom(21, True))
        self.httpserver.filelist[id] = (f, datetime.now(), ip)
        
        if isinstance(f, types.StringTypes):
            filename = os.path.split(f)[1]
        else:
            filename = f.filename
        
        return 'http://%s:%s/%s/%s' % (self.ip, self.port, id, urllib.quote_plus(filename))
    
    def get_uri(self, uri):
        f = self.filesystem.get_file(uri.real_path)
        if f is None:
            raise FileOrFolderNotFoundException("'%s' not found" % uri.real_path)
        
        if 'ip' in uri.parameters:
            ip = uri.parameters['ip']
        else:
            ip = None
        
        return self.prepare_serve(f, ip)
    
    def initialize(self):
        self.ip = self.configfile.get('general', 'externalip')
        self.port = self.configfile.getint(self.config_section, 'port')
        self.httpserver = HTTPServer()
        self.listeningport = reactor.listenTCP(self.port, server.Site(self.httpserver))
        if self.port == 0:
            self.port = self.listeningport.getHost().port
    
httpurihandler = HTTPURIHandler()
