import os
import shutil

from twisted.trial import unittest
from twisted.internet import defer

from sdfs.fs import FileSystem

class PluginTest(unittest.TestCase):
    def setUp(self):
        self.tmpfolder = os.path.join(os.getcwd(), self.mktemp())
        os.mkdir(self.tmpfolder)
        
        self.dbfolder = dbfolder = os.path.join(os.getcwd(), self.mktemp())
        os.mkdir(dbfolder)
        
        self.pluginfolder = 'something-entertaining'
        self.actualpluginfolder = os.path.join(self.tmpfolder, self.pluginfolder)
        os.mkdir(self.actualpluginfolder)
        
        self.filesystem = FileSystem([(self.tmpfolder, '/')], os.path.join(dbfolder, 'db.db'))
        return self.filesystem.rescan()
    
    def testMALPlugin(self):
        f = open(os.path.join(self.actualpluginfolder, 'mal.nfo'), 'w')
        f.write('10')
        f.close()
        
        def done_scanning(ignored):
            metadata = self.filesystem.list_dir('/')['']
            self.failUnlessEqual(len(metadata), 1, "Too many or too few elements in metadata")
            metadata = metadata[0]
            self.failUnlessIn('plugin:mal:id', metadata, 'No metadata tag found for folder')
            self.failUnlessEqual(metadata['plugin:mal:id'], '10', 'Invalid MAL id found during scanning')
        
        return self.filesystem.rescan().addCallback(done_scanning)
    
    def testIMDBPlugin(self):
        f = open(os.path.join(self.actualpluginfolder, 'some-random-nfo.nfo'), 'w')
        f.write('http://www.imdb.com/title/tt0120737/')
        f.close()
        
        def done_scanning(ignored):
            metadata = self.filesystem.list_dir('/')['']
            self.failUnlessEqual(len(metadata), 1, "Too many or too few elements in metadata")
            metadata = metadata[0]
            self.failUnlessIn('plugin:imdb:id', metadata, 'No metadata tag found for folder')
            self.failUnlessEqual(metadata['plugin:imdb:id'], '120737', 'Invalid IMDB id found during scanning')
        
        return self.filesystem.rescan().addCallback(done_scanning)
    
    @defer.inlineCallbacks
    def tearDown(self):
        yield self.filesystem.close()
        shutil.rmtree(self.tmpfolder)
        shutil.rmtree(self.dbfolder)