import urllib
import hashlib
from urlparse import urlparse

__all__ = [
    'uriparse',
    'cleanup_path',
    'hash_path',
    'keyify',
]

def uriparse(uri):
    """
    Parses an `uri` correctly by enforcing http as scheme when urlparse
    is executed. Adds two new attributes.
       
    New attributes:
    
    - `real_scheme`: The real scheme from the uri.
    - `real_path`: The unquoted path from the uri.
    """
    uri = uri.decode('utf-8').encode('utf-8').split(u':')
    scheme = uri[0]
    uri[0] = 'http'
    uri = urlparse(':'.join(uri))
    setattr(uri, 'real_path', urllib.unquote(uri.path.encode('utf-8')))
    setattr(uri, 'real_scheme', scheme)
    
    if uri[4]:
        parameters = dict([part.split('=') for part in uri[4].split('&')])
    else:
        parameters = {}
        
    setattr(uri, 'parameters', parameters)
    
    return uri

def cleanup_path(path):
    if type(path) is list:
        return [cleanup_path(p) for p in path]
    else:
        return path.strip('/')

def hash_path(path):
    return hashlib.sha1(path).digest()

def keyify(key_type, path):
    return '%s%s' % (key_type, hash_path(cleanup_path(path)))